@@ -127,9 +127,8 @@ class TestCredentialRefreshRaces(unittest.TestCase):
         max_calls_allowed = math.ceil((end - start) / 2.0) + 1
         self.assertTrue(
             creds.refresh_counter <= max_calls_allowed,
-            "Too many cred refreshes, max: %s, actual: %s, "
-            "time_delta: %.4f"
-            % (max_calls_allowed, creds.refresh_counter, (end - start)),
+            f"Too many cred refreshes, max: {max_calls_allowed}, actual: "
+            f"{creds.refresh_counter}, time_delta: {end - start:.4f}",
         )
 
     def test_no_race_for_immediate_advisory_expiration(self):
@@ -206,9 +205,12 @@ class BaseAssumeRoleTest(BaseEnvVar):
 
     def create_random_credentials(self):
         return Credentials(
-            'fake-%s' % random_chars(15),
-            'fake-%s' % random_chars(35),
-            'fake-%s' % random_chars(45),
+            f'fake-{random_chars(15)}',
+            f'fake-{random_chars(35)}',
+            f'fake-{random_chars(45)}',
+            # The account_id gets resolved from the
+            # Arn in create_assume_role_response().
+            account_id='1234567890',
         )
 
     def assert_creds_equal(self, c1, c2):
@@ -241,9 +243,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
         credential_process = os.path.join(
             current_dir, 'utils', 'credentialprocess.py'
         )
-        self.credential_process = '{} {}'.format(
-            sys.executable, credential_process
-        )
+        self.credential_process = f'{sys.executable} {credential_process}'
 
     def mock_provider(self, provider_cls):
         mock_instance = mock.Mock(spec=provider_cls)
@@ -487,7 +487,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'source_profile = B\n'
             '[profile B]\n'
-            'credential_process = %s\n' % self.credential_process
+            f'credential_process = {self.credential_process}\n'
         )
         self.write_config(config)
 
@@ -520,7 +520,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'source_profile = B\n'
             '[profile B]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleB\n'
-            'web_identity_token_file = %s\n' % token_path
+            f'web_identity_token_file = {token_path}\n'
         )
         self.write_config(config)
 
@@ -561,7 +561,7 @@ class TestAssumeRole(BaseAssumeRoleTest):
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'source_profile = B\n'
             '[profile B]\n'
-            'web_identity_token_file = %s\n' % token_path
+            f'web_identity_token_file = {token_path}\n'
         )
         self.write_config(config)
 
@@ -772,6 +772,26 @@ class TestAssumeRole(BaseAssumeRoleTest):
         self.assert_creds_equal(creds, expected_creds)
         self.assertEqual(self.actual_client_region, 'cn-north-1')
 
+    def test_assume_role_resolves_account_id(self):
+        config = (
+            '[profile A]\n'
+            'role_arn = arn:aws:iam::1234567890:role/RoleA\n'
+            'source_profile = B\n\n'
+            '[profile B]\n'
+            'aws_access_key_id = foo\n'
+            'aws_secret_access_key = bar\n'
+        )
+        self.write_config(config)
+        expected_creds = self.create_random_credentials()
+        response = self.create_assume_role_response(expected_creds)
+
+        session, stubber = self.create_session(profile='A')
+        stubber.add_response('assume_role', response)
+
+        actual_creds = session.get_credentials()
+        self.assert_creds_equal(actual_creds, expected_creds)
+        self.assertEqual(actual_creds.account_id, '1234567890')
+
 
 class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
     def setUp(self):
@@ -803,8 +823,8 @@ class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
             '[profile A]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'role_session_name = sname\n'
-            'web_identity_token_file = %s\n'
-        ) % self.token_file
+            f'web_identity_token_file = {self.token_file}\n'
+        )
         self.write_config(config)
         expected_params = {
             'RoleArn': 'arn:aws:iam::123456789:role/RoleA',
@@ -832,8 +852,8 @@ class TestAssumeRoleWithWebIdentity(BaseAssumeRoleTest):
             '[profile A]\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
             'role_session_name = aname\n'
-            'web_identity_token_file = %s\n'
-        ) % self.token_file
+            f'web_identity_token_file = {self.token_file}\n'
+        )
         self.write_config(config)
 
         different_token = os.path.join(self.tempdir, str(uuid.uuid4()))
@@ -856,9 +876,7 @@ class TestProcessProvider(unittest.TestCase):
         credential_process = os.path.join(
             current_dir, 'utils', 'credentialprocess.py'
         )
-        self.credential_process = '{} {}'.format(
-            sys.executable, credential_process
-        )
+        self.credential_process = f'{sys.executable} {credential_process}'
         self.environ = os.environ.copy()
         self.environ_patch = mock.patch('os.environ', self.environ)
         self.environ_patch.start()
@@ -921,8 +939,8 @@ class TestSTSRegional(BaseAssumeRoleTest):
     def _get_assume_role_body(self, method_name):
         expiration = self.some_future_time()
         body = (
-            '<{method_name}Response>'
-            '  <{method_name}Result>'
+            f'<{method_name}Response>'
+            f'  <{method_name}Result>'
             '    <AssumedRoleUser>'
             '      <Arn>arn:aws:sts::0123456:user</Arn>'
             '      <AssumedRoleId>AKID:mysession-1567020004</AssumedRoleId>'
@@ -931,11 +949,11 @@ class TestSTSRegional(BaseAssumeRoleTest):
             '      <AccessKeyId>AccessKey</AccessKeyId>'
             '      <SecretAccessKey>SecretKey</SecretAccessKey>'
             '      <SessionToken>SessionToken</SessionToken>'
-            '      <Expiration>{expiration}</Expiration>'
+            f'      <Expiration>{expiration}</Expiration>'
             '    </Credentials>'
-            '  </{method_name}Result>'
-            '</{method_name}Response>'
-        ).format(method_name=method_name, expiration=expiration)
+            f'  </{method_name}Result>'
+            f'</{method_name}Response>'
+        )
         return body.encode('utf-8')
 
     def make_stubbed_client_call_to_region(self, session, stubber, region):
@@ -976,11 +994,11 @@ class TestSTSRegional(BaseAssumeRoleTest):
             '[profile A]\n'
             'sts_regional_endpoints = regional\n'
             'role_arn = arn:aws:iam::123456789:role/RoleA\n'
-            'web_identity_token_file = %s\n'
+            f'web_identity_token_file = {token_file}\n'
             'source_profile = B\n\n'
             '[profile B]\n'
             'aws_access_key_id = abc123\n'
-            'aws_secret_access_key = def456\n' % token_file
+            'aws_secret_access_key = def456\n'
         )
         self.write_config(config)
         # Make an arbitrary client and API call as we are really only
@@ -1143,3 +1161,24 @@ class SSOSessionTest(BaseEnvVar):
             }
         }
         stubber.add_response(body=json.dumps(response).encode('utf-8'))
+
+
+class TestContextCredentials(unittest.TestCase):
+    ACCESS_KEY = "access-key"
+    SECRET_KEY = "secret-key"
+
+    def _add_fake_creds(self, request, **kwargs):
+        request.context.setdefault('signing', {})
+        request.context['signing']['request_credentials'] = Credentials(
+            self.ACCESS_KEY, self.SECRET_KEY
+        )
+
+    def test_credential_context_override(self):
+        session = StubbedSession()
+        with SessionHTTPStubber(session) as stubber:
+            s3 = session.create_client('s3')
+            s3.meta.events.register('before-sign', self._add_fake_creds)
+            stubber.add_response()
+            s3.list_buckets()
+            request = stubber.requests[0]
+            assert self.ACCESS_KEY in str(request.headers.get('Authorization'))
