@@ -10,28 +10,27 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-from datetime import datetime
 import re
+from datetime import datetime
 
-
-from tests import BaseSessionTest
-from tests import mock
-from tests import temporary_file
-from tests import assert_url_equal
-from tests import ClientHTTPStubber
-from botocore.stub import Stubber
 from botocore.config import Config
-
+from botocore.stub import Stubber
+from tests import (
+    BaseSessionTest,
+    ClientHTTPStubber,
+    assert_url_equal,
+    mock,
+    temporary_file,
+)
 
 _V4_SIGNING_REGION_REGEX = re.compile(
-    r'AWS4-HMAC-SHA256 '
-    r'Credential=\w+/\d+/(?P<signing_region>[a-z0-9-]+)/'
+    r'AWS4-HMAC-SHA256 ' r'Credential=\w+/\d+/(?P<signing_region>[a-z0-9-]+)/'
 )
 
 
 class TestSTSPresignedUrl(BaseSessionTest):
     def setUp(self):
-        super(TestSTSPresignedUrl, self).setUp()
+        super().setUp()
         self.client = self.session.create_client('sts', 'us-west-2')
         # Makes sure that no requests will go through
         self.stubber = Stubber(self.client)
@@ -56,23 +55,25 @@ class TestSTSPresignedUrl(BaseSessionTest):
 
 
 class TestSTSEndpoints(BaseSessionTest):
-    def create_sts_client(self, region, endpoint_url=None, use_ssl=True,
-                          config=None):
+    def create_sts_client(
+        self, region, endpoint_url=None, use_ssl=True, config=None
+    ):
         return self.session.create_client(
-            'sts', region_name=region, endpoint_url=endpoint_url,
-            use_ssl=use_ssl, config=config
+            'sts',
+            region_name=region,
+            endpoint_url=endpoint_url,
+            use_ssl=use_ssl,
+            config=config,
         )
 
     def set_sts_regional_for_config_file(self, fileobj, config_val):
-        fileobj.write(
-            '[default]\n'
-            'sts_regional_endpoints=%s\n' % config_val
-        )
+        fileobj.write('[default]\n' f'sts_regional_endpoints={config_val}\n')
         fileobj.flush()
         self.environ['AWS_CONFIG_FILE'] = fileobj.name
 
-    def assert_request_sent(self, sts, expected_url,
-                            expected_signing_region=None):
+    def assert_request_sent(
+        self, sts, expected_url, expected_signing_region=None
+    ):
         body = (
             b'<GetCallerIdentityResponse>'
             b'  <GetCallerIdentityResult>'
@@ -93,7 +94,7 @@ class TestSTSEndpoints(BaseSessionTest):
             if expected_signing_region:
                 self.assertEqual(
                     self._get_signing_region(captured_request),
-                    expected_signing_region
+                    expected_signing_region,
                 )
 
     def _get_signing_region(self, request):
@@ -107,7 +108,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.amazonaws.com/',
-            expected_signing_region='us-east-1'
+            expected_signing_region='us-east-1',
         )
 
     def test_legacy_region_with_regional_configured(self):
@@ -116,7 +117,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.us-west-2.amazonaws.com/',
-            expected_signing_region='us-west-2'
+            expected_signing_region='us-west-2',
         )
 
     def test_fips_endpoint_with_legacy_configured(self):
@@ -125,7 +126,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts-fips.us-west-2.amazonaws.com/',
-            expected_signing_region='us-west-2'
+            expected_signing_region='us-west-2',
         )
 
     def test_fips_endpoint_with_regional_configured(self):
@@ -134,7 +135,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts-fips.us-west-2.amazonaws.com/',
-            expected_signing_region='us-west-2'
+            expected_signing_region='us-west-2',
         )
 
     def test_dualstack_endpoint_with_legacy_configured(self):
@@ -144,7 +145,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.us-west-2.api.aws/',
-            expected_signing_region='us-west-2'
+            expected_signing_region='us-west-2',
         )
 
     def test_dualstack_endpoint_with_regional_configured(self):
@@ -154,7 +155,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.us-west-2.api.aws/',
-            expected_signing_region='us-west-2'
+            expected_signing_region='us-west-2',
         )
 
     def test_nonlegacy_region_with_legacy_configured(self):
@@ -163,7 +164,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.ap-east-1.amazonaws.com/',
-            expected_signing_region='ap-east-1'
+            expected_signing_region='ap-east-1',
         )
 
     def test_nonlegacy_region_with_regional_configured(self):
@@ -172,7 +173,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.ap-east-1.amazonaws.com/',
-            expected_signing_region='ap-east-1'
+            expected_signing_region='ap-east-1',
         )
 
     def test_nonaws_partition_region_with_legacy_configured(self):
@@ -181,7 +182,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.cn-north-1.amazonaws.com.cn/',
-            expected_signing_region='cn-north-1'
+            expected_signing_region='cn-north-1',
         )
 
     def test_nonaws_partition_region_with_regional_configured(self):
@@ -190,7 +191,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.cn-north-1.amazonaws.com.cn/',
-            expected_signing_region='cn-north-1'
+            expected_signing_region='cn-north-1',
         )
 
     def test_global_region_with_legacy_configured(self):
@@ -208,7 +209,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.amazonaws.com/',
-            expected_signing_region='us-east-1'
+            expected_signing_region='us-east-1',
         )
 
     def test_defaults_to_global_endpoint_for_legacy_region(self):
@@ -216,7 +217,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.amazonaws.com/',
-            expected_signing_region='us-east-1'
+            expected_signing_region='us-east-1',
         )
 
     def test_defaults_to_regional_endpoint_for_nonlegacy_region(self):
@@ -224,7 +225,7 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.ap-east-1.amazonaws.com/',
-            expected_signing_region='ap-east-1'
+            expected_signing_region='ap-east-1',
         )
 
     def test_configure_sts_regional_from_config_file(self):
@@ -242,35 +243,34 @@ class TestSTSEndpoints(BaseSessionTest):
             self.set_sts_regional_for_config_file(f, 'regional')
             sts = self.create_sts_client('us-west-2')
             self.assert_request_sent(
-                sts, expected_url='https://sts.amazonaws.com/')
+                sts, expected_url='https://sts.amazonaws.com/'
+            )
 
     def test_user_provided_endpoint_with_legacy_configured(self):
         self.environ['AWS_STS_REGIONAL_ENDPOINTS'] = 'legacy'
         sts = self.create_sts_client(
-            'us-west-2', endpoint_url='https://custom.com')
-        self.assert_request_sent(
-            sts, expected_url='https://custom.com/')
+            'us-west-2', endpoint_url='https://custom.com'
+        )
+        self.assert_request_sent(sts, expected_url='https://custom.com/')
 
     def test_user_provided_endpoint_with_regional_configured(self):
         self.environ['AWS_STS_REGIONAL_ENDPOINTS'] = 'regional'
         sts = self.create_sts_client(
-            'us-west-2', endpoint_url='https://custom.com')
-        self.assert_request_sent(
-            sts, expected_url='https://custom.com/')
+            'us-west-2', endpoint_url='https://custom.com'
+        )
+        self.assert_request_sent(sts, expected_url='https://custom.com/')
 
     def test_http_with_legacy_configured(self):
         self.environ['AWS_STS_REGIONAL_ENDPOINTS'] = 'legacy'
-        sts = self.create_sts_client(
-            'us-west-2', use_ssl=False)
-        self.assert_request_sent(
-            sts, expected_url='http://sts.amazonaws.com/')
+        sts = self.create_sts_client('us-west-2', use_ssl=False)
+        self.assert_request_sent(sts, expected_url='http://sts.amazonaws.com/')
 
     def test_client_for_unknown_region(self):
         sts = self.create_sts_client('not-real')
         self.assert_request_sent(
             sts,
             expected_url='https://sts.not-real.amazonaws.com/',
-            expected_signing_region='not-real'
+            expected_signing_region='not-real',
         )
 
     def test_client_for_unknown_region_with_legacy_configured(self):
@@ -286,5 +286,31 @@ class TestSTSEndpoints(BaseSessionTest):
         self.assert_request_sent(
             sts,
             expected_url='https://sts.not-real.amazonaws.com/',
-            expected_signing_region='not-real'
+            expected_signing_region='not-real',
         )
+
+
+def test_assume_role_with_saml_no_region_custom_endpoint(patched_session):
+    # When an endpoint_url and no region are given, AssumeRoleWithSAML should
+    # resolve to the endpoint_url and succeed, not fail in endpoint resolution:
+    # https://github.com/aws/aws-cli/issues/7455
+
+    client = patched_session.create_client(
+        'sts', region_name=None, endpoint_url="https://custom.endpoint.aws"
+    )
+    assert client.meta.region_name is None
+
+    mock_response_body = b"""\
+<AssumeRoleWithSAMLResponse xmlns="https://sts.amazonaws.com/doc/2011-06-15/">
+    <AssumeRoleWithSAMLResult></AssumeRoleWithSAMLResult>
+</AssumeRoleWithSAMLResponse>
+"""
+    with ClientHTTPStubber(client) as http_stubber:
+        http_stubber.add_response(body=mock_response_body)
+        client.assume_role_with_saml(
+            RoleArn='arn:aws:iam::123456789:role/RoleA',
+            PrincipalArn='arn:aws:iam::123456789:role/RoleB',
+            SAMLAssertion='xxxx',
+        )
+    captured_request = http_stubber.requests[0]
+    assert captured_request.url == "https://custom.endpoint.aws/"
