@@ -75,6 +75,7 @@ def setup_module():
         'CreateBucketConfiguration': {
             'LocationConstraint': _DEFAULT_REGION,
         },
+        'ObjectOwnership': 'ObjectWriter',
     }
     try:
         s3.create_bucket(**params)
@@ -84,6 +85,7 @@ def setup_module():
         # final call as to whether or not the bucket exists.
         LOG.debug("create_bucket() raised an exception: %s", e, exc_info=True)
     waiter.wait(Bucket=_SHARED_BUCKET)
+    s3.delete_public_access_block(Bucket=_SHARED_BUCKET)
 
 
 def clear_out_bucket(bucket, region, delete_bucket=False):
@@ -123,7 +125,6 @@ def teardown_module():
 
 
 class BaseS3ClientTest(unittest.TestCase):
-
     DEFAULT_DELAY = 5
 
     def setUp(self):
@@ -142,7 +143,10 @@ class BaseS3ClientTest(unittest.TestCase):
         bucket_client = client or self.client
         if bucket_name is None:
             bucket_name = random_bucketname()
-        bucket_kwargs = {'Bucket': bucket_name}
+        bucket_kwargs = {
+            'Bucket': bucket_name,
+            'ObjectOwnership': 'ObjectWriter',
+        }
         if region_name != 'us-east-1':
             bucket_kwargs['CreateBucketConfiguration'] = {
                 'LocationConstraint': region_name,
@@ -156,6 +160,7 @@ class BaseS3ClientTest(unittest.TestCase):
         consistency_waiter.wait(
             lambda: waiter.wait(Bucket=bucket_name) is None
         )
+        bucket_client.delete_public_access_block(Bucket=bucket_name)
         self.addCleanup(clear_out_bucket, bucket_name, region_name, True)
         return bucket_name
 
@@ -283,9 +288,7 @@ class TestS3BaseWithBucket(BaseS3ClientTest):
                 # Sleep and try again.
                 time.sleep(2)
         self.fail(
-            "Expected to see {} uploads, instead saw: {}".format(
-                num_uploads, amount_seen
-            )
+            f"Expected to see {num_uploads} uploads, instead saw: {amount_seen}"
         )
 
     def create_client(self):
@@ -354,7 +357,7 @@ class TestS3Objects(TestS3BaseWithBucket):
     @pytest.mark.slow
     def test_can_paginate(self):
         for i in range(5):
-            key_name = 'key%s' % i
+            key_name = f'key{i}'
             self.create_object(key_name)
         # Eventual consistency.
         time.sleep(3)
@@ -368,7 +371,7 @@ class TestS3Objects(TestS3BaseWithBucket):
     @pytest.mark.slow
     def test_can_paginate_with_page_size(self):
         for i in range(5):
-            key_name = 'key%s' % i
+            key_name = f'key{i}'
             self.create_object(key_name)
         # Eventual consistency.
         time.sleep(3)
@@ -387,7 +390,7 @@ class TestS3Objects(TestS3BaseWithBucket):
         for i in range(5):
             key_name = f'key/{i}/{i}'
             self.create_object(key_name)
-            key_name2 = 'key/%s' % i
+            key_name2 = f'key/{i}'
             self.create_object(key_name2)
         time.sleep(3)
         paginator = self.client.get_paginator('list_objects')
@@ -571,7 +574,7 @@ class TestS3Objects(TestS3BaseWithBucket):
         threads = []
         for i in range(10):
             t = threading.Thread(
-                target=self.create_object_catch_exceptions, args=('foo%s' % i,)
+                target=self.create_object_catch_exceptions, args=(f'foo{i}',)
             )
             t.daemon = True
             threads.append(t)
@@ -582,13 +585,12 @@ class TestS3Objects(TestS3BaseWithBucket):
         self.assertEqual(
             self.caught_exceptions,
             [],
-            "Unexpectedly caught exceptions: %s" % self.caught_exceptions,
+            f"Unexpectedly caught exceptions: {self.caught_exceptions}",
         )
         self.assertEqual(
             len(set(self.auth_paths)),
             10,
-            "Expected 10 unique auth paths, instead received: %s"
-            % (self.auth_paths),
+            f"Expected 10 unique auth paths, instead received: {self.auth_paths}",
         )
 
     def test_non_normalized_key_paths(self):
@@ -692,9 +694,7 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
     def setUp(self):
         super().setUp()
         self.region = 'us-east-1'
-        self.client_config = Config(
-            region_name=self.region, signature_version='s3'
-        )
+        self.client_config = Config(region_name=self.region)
         self.client = self.session.create_client(
             's3', config=self.client_config
         )
@@ -702,17 +702,19 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
         self.setup_bucket()
 
     def test_presign_sigv2(self):
+        self.client_config.signature_version = 's3'
+        self.client = self.session.create_client(
+            's3', config=self.client_config
+        )
         presigned_url = self.client.generate_presigned_url(
             'get_object', Params={'Bucket': self.bucket_name, 'Key': self.key}
         )
         self.assertTrue(
             presigned_url.startswith(
-                'https://{}.s3.amazonaws.com/{}'.format(
-                    self.bucket_name, self.key
-                )
+                f'https://{self.bucket_name}.s3.amazonaws.com/{self.key}'
             ),
             "Host was suppose to use DNS style, instead "
-            "got: %s" % presigned_url,
+            f"got: {presigned_url}",
         )
         # Try to retrieve the object using the presigned url.
         self.assertEqual(http_get(presigned_url).data, b'foo')
@@ -743,17 +745,19 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
         )
         self.assertTrue(
             presigned_url.startswith(
-                'https://{}.s3.amazonaws.com/{}'.format(
-                    self.bucket_name, self.key
-                )
+                f'https://{self.bucket_name}.s3.amazonaws.com/{self.key}'
             ),
             "Host was suppose to be the us-east-1 endpoint, instead "
-            "got: %s" % presigned_url,
+            f"got: {presigned_url}",
         )
         # Try to retrieve the object using the presigned url.
         self.assertEqual(http_get(presigned_url).data, b'foo')
 
     def test_presign_post_sigv2(self):
+        self.client_config.signature_version = 's3'
+        self.client = self.session.create_client(
+            's3', config=self.client_config
+        )
 
         # Create some of the various supported conditions.
         conditions = [
@@ -776,10 +780,11 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
         # Make sure the correct endpoint is being used
         self.assertTrue(
             post_args['url'].startswith(
-                'https://%s.s3.amazonaws.com' % self.bucket_name
+                f'https://{self.bucket_name}.s3.amazonaws.com'
+            ),
+            "Host was suppose to use DNS style, instead " "got: {}".format(
+                post_args['url']
             ),
-            "Host was suppose to use DNS style, instead "
-            "got: %s" % post_args['url'],
         )
 
         # Try to retrieve the object using the presigned url.
@@ -813,10 +818,10 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
         # Make sure the correct endpoint is being used
         self.assertTrue(
             post_args['url'].startswith(
-                'https://%s.s3.amazonaws.com/' % self.bucket_name
+                f'https://{self.bucket_name}.s3.amazonaws.com/'
             ),
             "Host was suppose to use us-east-1 endpoint, instead "
-            "got: %s" % post_args['url'],
+            "got: {}".format(post_args['url']),
         )
 
         r = http_post(post_args['url'], data=post_args['fields'], files=files)
@@ -826,26 +831,27 @@ class TestS3PresignUsStandard(BaseS3PresignTest):
 class TestS3PresignNonUsStandard(BaseS3PresignTest):
     def setUp(self):
         super().setUp()
-        self.client_config = Config(
-            region_name=self.region, signature_version='s3'
-        )
+        self.client_config = Config(region_name=self.region)
         self.client = self.session.create_client(
             's3', config=self.client_config
         )
         self.setup_bucket()
 
     def test_presign_sigv2(self):
+        self.client_config.signature_version = 's3'
+        self.client = self.session.create_client(
+            's3', config=self.client_config
+        )
+
         presigned_url = self.client.generate_presigned_url(
             'get_object', Params={'Bucket': self.bucket_name, 'Key': self.key}
         )
         self.assertTrue(
             presigned_url.startswith(
-                'https://{}.s3.amazonaws.com/{}'.format(
-                    self.bucket_name, self.key
-                )
+                f'https://{self.bucket_name}.s3.amazonaws.com/{self.key}'
             ),
             "Host was suppose to use DNS style, instead "
-            "got: %s" % presigned_url,
+            f"got: {presigned_url}",
         )
         # Try to retrieve the object using the presigned url.
         self.assertEqual(http_get(presigned_url).data, b'foo')
@@ -868,17 +874,20 @@ class TestS3PresignNonUsStandard(BaseS3PresignTest):
 
         self.assertTrue(
             presigned_url.startswith(
-                'https://s3.us-west-2.amazonaws.com/{}/{}'.format(
-                    self.bucket_name, self.key
-                )
+                f'https://s3.us-west-2.amazonaws.com/{self.bucket_name}/{self.key}'
             ),
             "Host was suppose to be the us-west-2 endpoint, instead "
-            "got: %s" % presigned_url,
+            f"got: {presigned_url}",
         )
         # Try to retrieve the object using the presigned url.
         self.assertEqual(http_get(presigned_url).data, b'foo')
 
     def test_presign_post_sigv2(self):
+        self.client_config.signature_version = 's3'
+        self.client = self.session.create_client(
+            's3', config=self.client_config
+        )
+
         # Create some of the various supported conditions.
         conditions = [
             {"acl": "public-read"},
@@ -900,10 +909,11 @@ class TestS3PresignNonUsStandard(BaseS3PresignTest):
         # Make sure the correct endpoint is being used
         self.assertTrue(
             post_args['url'].startswith(
-                'https://%s.s3.amazonaws.com' % self.bucket_name
+                f'https://{self.bucket_name}.s3.amazonaws.com'
+            ),
+            "Host was suppose to use DNS style, instead " "got: {}".format(
+                post_args['url']
             ),
-            "Host was suppose to use DNS style, instead "
-            "got: %s" % post_args['url'],
         )
 
         r = http_post(post_args['url'], data=post_args['fields'], files=files)
@@ -936,10 +946,11 @@ class TestS3PresignNonUsStandard(BaseS3PresignTest):
         # Make sure the correct endpoint is being used
         self.assertTrue(
             post_args['url'].startswith(
-                'https://%s.s3.amazonaws.com/' % self.bucket_name
+                f'https://{self.bucket_name}.s3.amazonaws.com/'
+            ),
+            "Host was suppose to use DNS style, instead " "got: {}".format(
+                post_args['url']
             ),
-            "Host was suppose to use DNS style, instead "
-            "got: %s" % post_args['url'],
         )
 
         r = http_post(post_args['url'], data=post_args['fields'], files=files)
@@ -1402,7 +1413,7 @@ class TestRegionRedirect(BaseS3ClientTest):
             )
             self.assertEqual(response.get('ContentLength'), len(key))
         except ClientError as e:
-            self.fail("S3 Client failed to redirect Head Object: %s" % e)
+            self.fail(f"S3 Client failed to redirect Head Object: {e}")
 
 
 class TestBucketWithVersions(BaseS3ClientTest):
