@@ -14,13 +14,19 @@ import unittest
 from io import BytesIO
 
 from botocore.awsrequest import AWSResponse
-from botocore.exceptions import AwsChunkedWrapperError, FlexibleChecksumError
+from botocore.compat import HAS_CRT
+from botocore.config import Config
+from botocore.exceptions import (
+    AwsChunkedWrapperError,
+    FlexibleChecksumError,
+    MissingDependencyException,
+)
 from botocore.httpchecksum import (
-    _CHECKSUM_CLS,
     AwsChunkedWrapper,
     Crc32Checksum,
     CrtCrc32cChecksum,
     CrtCrc32Checksum,
+    CrtCrc64NvmeChecksum,
     Sha1Checksum,
     Sha256Checksum,
     StreamingChecksumBody,
@@ -29,8 +35,8 @@ from botocore.httpchecksum import (
     resolve_request_checksum_algorithm,
     resolve_response_checksum_algorithms,
 )
-from botocore.model import OperationModel
-from tests import mock, requires_crt
+from botocore.model import OperationModel, StringShape, StructureShape
+from tests import get_checksum_cls, mock, requires_crt
 
 
 class TestHttpChecksumHandlers(unittest.TestCase):
@@ -48,6 +54,13 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         operation.http_checksum_required = required
         operation.has_streaming_output = streaming_output
         operation.has_streaming_input = streaming_input
+        if http_checksum and "requestAlgorithmMember" in http_checksum:
+            shape = mock.Mock(spec=StringShape)
+            shape.serialization = {"name": "x-amz-request-algorithm"}
+            operation.input_shape = mock.Mock(spec=StructureShape)
+            operation.input_shape.members = {
+                http_checksum["requestAlgorithmMember"]: shape
+            }
         return operation
 
     def _make_http_response(
@@ -83,7 +96,11 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         request = {
             "headers": {},
             "body": body,
-            "context": {},
+            "context": {
+                "client_config": Config(
+                    request_checksum_calculation="when_supported",
+                )
+            },
             "url": "https://example.com",
         }
         return request
@@ -95,26 +112,42 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         resolve_request_checksum_algorithm(request, operation_model, params)
         self.assertNotIn("checksum", request["context"])
 
-    def test_request_checksum_algorithm_model_opt_in(self):
+    def test_request_checksum_algorithm_model_default(self):
         operation_model = self._make_operation_model(
             http_checksum={"requestAlgorithmMember": "Algorithm"}
         )
 
-        # Param is not present, no checksum will be set
+        # Param is not present, crc32 checksum will be set by default
         params = {}
         request = self._build_request(b"")
         resolve_request_checksum_algorithm(request, operation_model, params)
-        self.assertNotIn("checksum", request["context"])
-
-        # Param is present, crc32 checksum will be set
-        params = {"Algorithm": "crc32"}
-        request = self._build_request(b"")
-        resolve_request_checksum_algorithm(request, operation_model, params)
         expected_algorithm = {
             "algorithm": "crc32",
             "in": "header",
             "name": "x-amz-checksum-crc32",
         }
+        expected_request_algorithm_header = {
+            "name": "x-amz-request-algorithm",
+            "value": "CRC32",
+        }
+        actual_algorithm = request["context"]["checksum"]["request_algorithm"]
+        actual_request_algorithm_header = request["context"]["checksum"][
+            "request_algorithm_header"
+        ]
+        self.assertEqual(actual_algorithm, expected_algorithm)
+        self.assertEqual(
+            actual_request_algorithm_header, expected_request_algorithm_header
+        )
+
+        # Param is present, sha256 checksum will be set
+        params = {"Algorithm": "sha256"}
+        request = self._build_request(b"")
+        resolve_request_checksum_algorithm(request, operation_model, params)
+        expected_algorithm = {
+            "algorithm": "sha256",
+            "in": "header",
+            "name": "x-amz-checksum-sha256",
+        }
         actual_algorithm = request["context"]["checksum"]["request_algorithm"]
         self.assertEqual(actual_algorithm, expected_algorithm)
 
@@ -125,32 +158,143 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         resolve_request_checksum_algorithm(request, operation_model, params)
         self.assertNotIn("checksum", request["context"])
 
-    def test_request_checksum_algorithm_model_opt_in_streaming(self):
+    def test_request_checksum_algorithm_model_default_streaming(self):
         request = self._build_request(b"")
         operation_model = self._make_operation_model(
             http_checksum={"requestAlgorithmMember": "Algorithm"},
             streaming_input=True,
         )
 
-        # Param is not present, no checksum will be set
+        # Param is not present, crc32 checksum will be set
         params = {}
         resolve_request_checksum_algorithm(request, operation_model, params)
-        self.assertNotIn("checksum", request["context"])
-
-        # Param is present, crc32 checksum will be set in the trailer
-        params = {"Algorithm": "crc32"}
-        resolve_request_checksum_algorithm(request, operation_model, params)
         expected_algorithm = {
             "algorithm": "crc32",
             "in": "trailer",
             "name": "x-amz-checksum-crc32",
         }
+        expected_request_algorithm_header = {
+            "name": "x-amz-request-algorithm",
+            "value": "CRC32",
+        }
+        actual_algorithm = request["context"]["checksum"]["request_algorithm"]
+        actual_request_algorithm_header = request["context"]["checksum"][
+            "request_algorithm_header"
+        ]
+        self.assertEqual(actual_algorithm, expected_algorithm)
+        self.assertEqual(
+            actual_request_algorithm_header, expected_request_algorithm_header
+        )
+
+        # Param is present, sha256 checksum will be set in the trailer
+        params = {"Algorithm": "sha256"}
+        resolve_request_checksum_algorithm(request, operation_model, params)
+        expected_algorithm = {
+            "algorithm": "sha256",
+            "in": "trailer",
+            "name": "x-amz-checksum-sha256",
+        }
         actual_algorithm = request["context"]["checksum"]["request_algorithm"]
         self.assertEqual(actual_algorithm, expected_algorithm)
 
         # Trailer should not be used for http endpoints
         request = self._build_request(b"")
         request["url"] = "http://example.com"
+        resolve_request_checksum_algorithm(request, operation_model, params)
+        expected_algorithm = {
+            "algorithm": "sha256",
+            "in": "header",
+            "name": "x-amz-checksum-sha256",
+        }
+        actual_algorithm = request["context"]["checksum"]["request_algorithm"]
+        self.assertEqual(actual_algorithm, expected_algorithm)
+
+    def test_request_checksum_algorithm_s3_signature_version_input_streaming(
+        self,
+    ):
+        config = Config(signature_version="s3")
+        request = self._build_request(b"")
+        request["context"]["client_config"] = config
+        operation_model = self._make_operation_model(
+            http_checksum={"requestChecksumRequired": True},
+            streaming_input=True,
+        )
+
+        params = {}
+        resolve_request_checksum_algorithm(request, operation_model, params)
+        actual_algorithm = request["context"]["checksum"]["request_algorithm"]
+        # Operations with streaming input using the "s3" signature_version should send
+        # checksums in the header, not trailer.
+        expected_algorithm = {
+            "algorithm": "crc32",
+            "in": "header",
+            "name": "x-amz-checksum-crc32",
+        }
+        self.assertEqual(actual_algorithm, expected_algorithm)
+
+    def test_request_checksum_algorithm_presigned_request(self):
+        request = self._build_request(b"")
+        request["context"]["is_presign_request"] = True
+        operation_model = self._make_operation_model(
+            http_checksum={"requestChecksumRequired": True},
+        )
+
+        params = {}
+        resolve_request_checksum_algorithm(request, operation_model, params)
+        # Presigned requests shouldn't use flexible checksums by default.
+        self.assertNotIn("checksum", request["context"])
+
+    def test_request_checksum_algorithm_model_unsupported_algorithm(self):
+        request = self._build_request(b"")
+        operation_model = self._make_operation_model(
+            http_checksum={"requestAlgorithmMember": "Algorithm"},
+        )
+        params = {"Algorithm": "sha256"}
+
+        with self.assertRaises(FlexibleChecksumError):
+            resolve_request_checksum_algorithm(
+                request, operation_model, params, supported_algorithms=[]
+            )
+
+    @unittest.skipIf(HAS_CRT, "Error only expected when CRT is not available")
+    def test_request_checksum_algorithm_model_no_crt_crc32c_unsupported(self):
+        request = self._build_request(b"")
+        operation_model = self._make_operation_model(
+            http_checksum={"requestAlgorithmMember": "Algorithm"},
+        )
+        params = {"Algorithm": "crc32c"}
+        with self.assertRaises(MissingDependencyException) as context:
+            resolve_request_checksum_algorithm(
+                request, operation_model, params
+            )
+            self.assertIn(
+                "Using CRC32C requires an additional dependency",
+                str(context.exception),
+            )
+
+    @unittest.skipIf(HAS_CRT, "Error only expected when CRT is not available")
+    def test_request_checksum_algorithm_model_no_crt_crc64nvme_unsupported(
+        self,
+    ):
+        request = self._build_request(b"")
+        operation_model = self._make_operation_model(
+            http_checksum={"requestAlgorithmMember": "Algorithm"},
+        )
+        params = {"Algorithm": "crc64nvme"}
+        with self.assertRaises(MissingDependencyException) as context:
+            resolve_request_checksum_algorithm(
+                request, operation_model, params
+            )
+            self.assertIn(
+                "Using CRC64NVME requires an additional dependency",
+                str(context.exception),
+            )
+
+    def test_request_checksum_algorithm_model_legacy_crc32(self):
+        request = self._build_request(b"")
+        operation_model = self._make_operation_model(required=True)
+        params = {}
+
         resolve_request_checksum_algorithm(request, operation_model, params)
         expected_algorithm = {
             "algorithm": "crc32",
@@ -160,29 +304,7 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         actual_algorithm = request["context"]["checksum"]["request_algorithm"]
         self.assertEqual(actual_algorithm, expected_algorithm)
 
-    def test_request_checksum_algorithm_model_unsupported_algorithm(self):
-        request = self._build_request(b"")
-        operation_model = self._make_operation_model(
-            http_checksum={"requestAlgorithmMember": "Algorithm"},
-        )
-        params = {"Algorithm": "crc32"}
-
-        with self.assertRaises(FlexibleChecksumError):
-            resolve_request_checksum_algorithm(
-                request, operation_model, params, supported_algorithms=[]
-            )
-
-    def test_request_checksum_algorithm_model_legacy_md5(self):
-        request = self._build_request(b"")
-        operation_model = self._make_operation_model(required=True)
-        params = {}
-
-        resolve_request_checksum_algorithm(request, operation_model, params)
-        actual_algorithm = request["context"]["checksum"]["request_algorithm"]
-        expected_algorithm = "conditional-md5"
-        self.assertEqual(actual_algorithm, expected_algorithm)
-
-    def test_request_checksum_algorithm_model_new_md5(self):
+    def test_request_checksum_algorithm_model_new_crc32(self):
         request = self._build_request(b"")
         operation_model = self._make_operation_model(
             http_checksum={"requestChecksumRequired": True}
@@ -191,7 +313,11 @@ class TestHttpChecksumHandlers(unittest.TestCase):
 
         resolve_request_checksum_algorithm(request, operation_model, params)
         actual_algorithm = request["context"]["checksum"]["request_algorithm"]
-        expected_algorithm = "conditional-md5"
+        expected_algorithm = {
+            "algorithm": "crc32",
+            "in": "header",
+            "name": "x-amz-checksum-crc32",
+        }
         self.assertEqual(actual_algorithm, expected_algorithm)
 
     def test_apply_request_checksum_handles_no_checksum_context(self):
@@ -199,7 +325,9 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         apply_request_checksum(request)
         # Build another request and assert the original request is the same
         expected_request = self._build_request(b"")
-        self.assertEqual(request, expected_request)
+        self.assertEqual(request["headers"], expected_request["headers"])
+        self.assertEqual(request["body"], expected_request["body"])
+        self.assertEqual(request["url"], expected_request["url"])
 
     def test_apply_request_checksum_handles_invalid_context(self):
         request = self._build_request(b"")
@@ -213,14 +341,6 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         with self.assertRaises(FlexibleChecksumError):
             apply_request_checksum(request)
 
-    def test_apply_request_checksum_conditional_md5(self):
-        request = self._build_request(b"")
-        request["context"]["checksum"] = {
-            "request_algorithm": "conditional-md5"
-        }
-        apply_request_checksum(request)
-        self.assertIn("Content-MD5", request["headers"])
-
     def test_apply_request_checksum_flex_header_bytes(self):
         request = self._build_request(b"")
         request["context"]["checksum"] = {
@@ -301,6 +421,50 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         # The body should not have been wrapped
         self.assertIsInstance(request["body"], bytes)
 
+    def test_apply_request_checksum_content_encoding_preset(self):
+        request = self._build_request(b"")
+        request["context"]["checksum"] = {
+            "request_algorithm": {
+                "in": "trailer",
+                "algorithm": "crc32",
+                "name": "x-amz-checksum-crc32",
+            }
+        }
+        request["headers"]["Content-Encoding"] = "foo"
+        apply_request_checksum(request)
+        # The content encoding should only have been appended
+        self.assertEqual(
+            request["headers"]["Content-Encoding"], "foo,aws-chunked"
+        )
+
+    def test_apply_request_checksum_content_encoding_default(self):
+        request = self._build_request(b"")
+        request["context"]["checksum"] = {
+            "request_algorithm": {
+                "in": "trailer",
+                "algorithm": "crc32",
+                "name": "x-amz-checksum-crc32",
+            }
+        }
+        apply_request_checksum(request)
+        self.assertEqual(request["headers"]["Content-Encoding"], "aws-chunked")
+
+    def test_apply_request_checksum_extra_headers(self):
+        request = self._build_request(b"")
+        request["context"]["checksum"] = {
+            "request_algorithm": {
+                "in": "trailer",
+                "algorithm": "crc32",
+                "name": "x-amz-checksum-crc32",
+            },
+            "request_algorithm_header": {
+                "name": "foo",
+                "value": "bar",
+            },
+        }
+        apply_request_checksum(request)
+        self.assertEqual(request["headers"]["foo"], "bar")
+
     def test_response_checksum_algorithm_no_model(self):
         request = self._build_request(b"")
         operation_model = self._make_operation_model()
@@ -308,7 +472,7 @@ class TestHttpChecksumHandlers(unittest.TestCase):
         resolve_response_checksum_algorithms(request, operation_model, params)
         self.assertNotIn("checksum", request["context"])
 
-    def test_response_checksum_algorithm_model_opt_in(self):
+    def test_response_checksum_algorithm_model_default(self):
         request = self._build_request(b"")
         operation_model = self._make_operation_model(
             http_checksum={
@@ -642,18 +806,27 @@ class TestChecksumImplementations(unittest.TestCase):
     def test_crt_crc32c(self):
         self.assert_base64_checksum(CrtCrc32cChecksum, "yZRlqg==")
 
+    @requires_crt()
+    def test_crt_crc64nvme(self):
+        self.assert_base64_checksum(CrtCrc64NvmeChecksum, "jSnVw/bqjr4=")
+
 
 class TestCrtChecksumOverrides(unittest.TestCase):
     @requires_crt()
     def test_crt_crc32_available(self):
-        actual_cls = _CHECKSUM_CLS.get("crc32")
+        actual_cls = get_checksum_cls("crc32")
         self.assertEqual(actual_cls, CrtCrc32Checksum)
 
     @requires_crt()
     def test_crt_crc32c_available(self):
-        actual_cls = _CHECKSUM_CLS.get("crc32c")
+        actual_cls = get_checksum_cls("crc32c")
         self.assertEqual(actual_cls, CrtCrc32cChecksum)
 
+    @requires_crt()
+    def test_crt_crc64nvme_available(self):
+        actual_cls = get_checksum_cls("crc64nvme")
+        self.assertEqual(actual_cls, CrtCrc64NvmeChecksum)
+
 
 class TestStreamingChecksumBody(unittest.TestCase):
     def setUp(self):
