@@ -35,9 +35,14 @@ from botocore.signers import (
     CloudFrontSigner,
     RequestSigner,
     S3PostPresigner,
+    _dsql_generate_db_auth_token,
+    dsql_generate_db_connect_admin_auth_token,
+    dsql_generate_db_connect_auth_token,
     generate_db_auth_token,
 )
-from tests import assert_url_equal, mock, unittest
+from tests import FreezeTime, assert_url_equal, mock, unittest
+
+DATE = datetime.datetime(2024, 11, 7, 17, 39, 33, tzinfo=tzutc())
 
 
 @pytest.fixture
@@ -996,7 +1001,7 @@ class TestGenerateUrl(unittest.TestCase):
             self.assertTrue(
                 kwargs.get('context', {}).get('is_presign_request'),
                 'The context did not have is_presign_request set to True for '
-                'the following kwargs emitted: %s' % kwargs,
+                f'the following kwargs emitted: {kwargs}',
             )
 
     def test_context_param_from_event_handler_sent_to_endpoint_resolver(self):
@@ -1178,6 +1183,81 @@ class TestGenerateDBAuthToken(BaseSignerTest):
         self.assertIn(hostname, result)
 
 
+class TestDSQLGenerateDBAuthToken(BaseSignerTest):
+    def setUp(self):
+        self.session = botocore.session.get_session()
+        self.client = self.session.create_client(
+            'dsql',
+            region_name='us-east-1',
+            aws_access_key_id='ACCESS_KEY',
+            aws_secret_access_key='SECRET_KEY',
+            aws_session_token="SESSION_TOKEN",
+        )
+        self.hostname = 'test.dsql.us-east-1.on.aws'
+        self.action = 'DbConnect'
+
+    @FreezeTime(botocore.auth.datetime, date=DATE)
+    def test_dsql_generate_db_auth_token(self):
+        result = _dsql_generate_db_auth_token(
+            self.client, self.hostname, self.action
+        )
+
+        expected_result = (
+            'test.dsql.us-east-1.on.aws/?Action=DbConnect'
+            '&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential='
+            'ACCESS_KEY%2F20241107%2Fus-east-1%2Fdsql%2Faws4_request'
+            '&X-Amz-Date=20241107T173933Z&X-Amz-Expires=900&X-Amz-SignedHeaders=host'
+            '&X-Amz-Security-Token=SESSION_TOKEN&X-Amz-Signature='
+            '57fe03e060348aaa21405c239bf02572bbc911076e94dcd65c12ae569dd8fcf4'
+        )
+
+        # A scheme needs to be appended to the beginning or urlsplit may fail
+        # on certain systems.
+        assert_url_equal('https://' + result, 'https://' + expected_result)
+
+    @FreezeTime(botocore.auth.datetime, date=DATE)
+    def test_dsql_generate_db_connect_auth_token(self):
+        result = dsql_generate_db_connect_auth_token(
+            self.client, self.hostname
+        )
+
+        expected_result = (
+            'test.dsql.us-east-1.on.aws/?Action=DbConnect'
+            '&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential='
+            'ACCESS_KEY%2F20241107%2Fus-east-1%2Fdsql%2Faws4_request'
+            '&X-Amz-Date=20241107T173933Z&X-Amz-Expires=900&X-Amz-SignedHeaders=host'
+            '&X-Amz-Security-Token=SESSION_TOKEN&X-Amz-Signature='
+            '57fe03e060348aaa21405c239bf02572bbc911076e94dcd65c12ae569dd8fcf4'
+        )
+
+        # A scheme needs to be appended to the beginning or urlsplit may fail
+        # on certain systems.
+        assert_url_equal('https://' + result, 'https://' + expected_result)
+
+    @FreezeTime(botocore.auth.datetime, date=DATE)
+    def test_dsql_generate_db_connect_admin_auth_token(self):
+        result = dsql_generate_db_connect_admin_auth_token(
+            self.client, self.hostname
+        )
+
+        expected_result = (
+            'test.dsql.us-east-1.on.aws/?Action=DbConnectAdmin'
+            '&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential='
+            'ACCESS_KEY%2F20241107%2Fus-east-1%2Fdsql%2Faws4_request'
+            '&X-Amz-Date=20241107T173933Z&X-Amz-Expires=900&X-Amz-SignedHeaders=host'
+            '&X-Amz-Security-Token=SESSION_TOKEN&X-Amz-Signature='
+            '5ac084bc7cabccc19a52a5d1b5c24b50d3ce143f43b659bd484c91aaf555e190'
+        )
+
+        # A scheme needs to be appended to the beginning or urlsplit may fail
+        # on certain systems.
+        assert_url_equal('https://' + result, 'https://' + expected_result)
+
+    def test_dsql_generate_db_auth_token_invalid_action(self):
+        with self.assertRaises(ParamValidationError):
+            _dsql_generate_db_auth_token(self.client, self.hostname, "FooBar")
+
+
 @pytest.mark.parametrize(
     'request_method',
     ['GET', 'HEAD', 'OPTIONS', 'POST', 'PUT', 'DELETE', None],
